@@ -1,25 +1,41 @@
 #######################################################################
-# $Id: TriangularNumbers.pm,v 1.7 2005/03/06 19:41:01 dpchrist Exp $
+# $Id: TriangularNumbers.pm,v 1.12 2010-06-30 03:59:57 dpchrist Exp $
 
 =head1 NAME
 
 Math::TriangularNumbers - Perl extension for Triangular numbers.
 
-Version 0.03
-
 
 =head1 SYNOPSIS
 
-  use Math::TriangularNumbers qw(T Ti is_T);
-  
-  ##### prints Triangular numbers "1, 3, 6, 10, 15, 21"
-  print STDERR join(", ",T(1),T(2),T(3),T(4),T(5),T(6)), "\n";
+See the script t/example.t:
 
-  ##### prints right Triangular indices "1, 2, 2, 3, 3, 3":
-  print STDERR join(", ",Ti(1),Ti(2),Ti(3),Ti(4),Ti(5),Ti(6)), "\n";
-  
-  ##### determines if the number 666 is triangular (it is):
-  print STDERR is_T(666) ? "yes\n" : "no\n";
+    use Math::TriangularNumbers qw(T Ti Tli Tri is_T);
+
+    ##### triangular numbers 1, 3, 6, 10, 15, 21, 28, 36, 45, 55
+    print STDERR join(", ", map({T($_)}   1..10)), "\n";
+
+    ##### triangular indices: 1, 1.x, 2, 2.x, 2.x, 3, 3.x, 3.x, 3.x, 4
+    print STDERR join(", ", map({Ti($_)}  1..10)), "\n";
+
+    ##### left triangular indices: 1, 1, 2, 2, 2, 3, 3, 3, 3, 4
+    print STDERR join(", ", map({Tli($_)} 1..10)), "\n";
+
+    ##### right triangular indices 1, 2, 2, 3, 3, 3, 4, 4, 4, 4
+    print STDERR join(", ", map({Tri($_)} 1..10)), "\n";
+
+    ##### determines if the number 666 is triangular (it is):
+    print STDERR "the number 666 ",
+		is_T(666) ? "is " : "is not",
+		"triangular\n";
+
+Output:
+
+    1, 3, 6, 10, 15, 21, 28, 36, 45, 55
+    1, 1.56155281280883, 2, 2.37228132326901, 2.70156211871642, 3, 3.27491721763537, 3.53112887414927, 3.77200187265877, 4
+    1, 1, 2, 2, 2, 3, 3, 3, 3, 4
+    1, 2, 2, 3, 3, 3, 4, 4, 4, 4
+    the number 666 is triangular
 
 
 =head1 DESCRIPTION
@@ -54,16 +70,9 @@ These are called Triangular numbers, after Pascal's Triangle:
 Observe the diagonal numbers starting at the third row:
 1, 3, 6, 10, 15, 21.
 
-This module implements the function T(n), given by the above equation,
-and it's "right inverse", T'(N), given by:
+The inverse is given by:
 
-	T'(N) = int( (1 + sqrt(1 + 8*(N-1))) / 2 )
-
-E.g.  Given an integer N which is not necessarily a Triangular Number,
-find n such that T(n-1) < N <= T(n). 
-
-I have extended the definition of a Triangular number
-to include zero and negative indices and integers.
+    n = -0.5 +- sqr(0.25 + 2*T)
 
 
 =head2 EXPORT
@@ -82,12 +91,16 @@ package Math::TriangularNumbers;
 # uses/ requires:
 #----------------------------------------------------------------------
 
-use 5.006;
+use 5.010;
 use strict;
 use warnings;
 
+use constant DEBUG => 0;
+
 use Carp;
-use Data::Dumper;
+use POSIX qw(ceil floor);
+
+require Data::Dumper if DEBUG;
 
 require Exporter;
 
@@ -100,6 +113,8 @@ our @ISA = qw(Exporter);
 our %EXPORT_TAGS = ( 'all' => [ qw(
     T
     Ti
+    Tli
+    Tri
     is_T
 ) ] );
 
@@ -107,9 +122,7 @@ our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 
 our @EXPORT = qw();
 
-our $VERSION = '0.03';
-
-our $debug = 0;
+our $VERSION = sprintf("%d.%03d", q$Revision: 1.12 $ =~ /(\d+)/g);
 
 #######################################################################
 # preloaded methods:
@@ -120,7 +133,6 @@ our $debug = 0;
 =head3 T($n)
 
 Returns the $n-th Triangular number.
-Croaks if passed a non-integer argument.
 Note that T(0) = 0, and T(-$n) = -T($n).
 
 =cut
@@ -128,20 +140,17 @@ Note that T(0) = 0, and T(-$n) = -T($n).
 sub T
 {
     my $n = shift;
-    print STDERR (caller(0))[3], ": ",
+    print STDERR __FILE__, ' ', __LINE__, ' ', (caller(0))[3], ": ",
 	Data::Dumper->Dump([$n], [qw(n)]), "\n"
-	if $debug;
+	if DEBUG;
 
-    croak "'$n' is not an integer"
-	unless $n =~ /^-?\d+$/;
-    
     my $T = 0 < $n
 	?  $n * ( $n+1) / 2
 	:  $n * (-$n+1) / 2;		##### let T(-n) = -T(n)
     
-    print STDERR (caller(0))[3], ": ",
+    print STDERR __FILE__, ' ', __LINE__, ' ', (caller(0))[3], ": ",
 	Data::Dumper->Dump([$T], [qw(T)]), "\n"
-	if $debug;
+	if DEBUG;
     return $T;
 }
 
@@ -149,31 +158,83 @@ sub T
 
 =head3 Ti($N)
 
-Returns right Triangular index of $N.
-Croaks if passed a non-integer argument.
-Note that Ti(0) = 0, and Ti(-$T) = -Ti($T).
+Returns triangular inverse (floating point) of $N.
+Note that Ti(0) = 0, and Ti(-$N) = -Ti($N).
 
 =cut
 
 sub Ti
 {
     my $N = shift;
-    print STDERR (caller(0))[3], ": ",
+    print STDERR __FILE__, ' ', __LINE__, ' ', (caller(0))[3], ": ",
 	Data::Dumper->Dump([$N], [qw(N)]), "\n"
-	if $debug;
+	if DEBUG;
 
-    croak "'$N' is not an integer"
-	unless $N =~ /^-?\d+$/;
-    
     my $n = ($N == 0)
 	    ? 0
-	    : (0 < $N)
-	      ?  int((1 + sqrt(1 + 8*($N-1))) / 2)
-	      : -int((1 + sqrt(1 - 8*($N+1))) / 2); ### T'(-N) = -T'(N)
+	    : ($N < 0)
+	      ?  +0.5 - sqrt(0.25 - 2 * $N)
+	      :  -0.5 + sqrt(0.25 + 2 * $N);
 
-    print STDERR (caller(0))[3], ": ",
+    print STDERR __FILE__, ' ', __LINE__, ' ', (caller(0))[3], ": ",
 	Data::Dumper->Dump([$n], [qw(n)]), "\n"
-	if $debug;
+	if DEBUG;
+    return $n;
+}
+
+#----------------------------------------------------------------------
+
+=head3 Tli($N)
+
+Returns left triangular index (integer) of $N.
+Note that Tli(0) = 0, and Tli(-$N) = -Tli($N).
+
+=cut
+
+sub Tli
+{
+    my $N = shift;
+    print STDERR __FILE__, ' ', __LINE__, ' ', (caller(0))[3], ": ",
+	Data::Dumper->Dump([$N], [qw(N)]), "\n"
+	if DEBUG;
+
+    my $n = ($N == 0)
+	    ? 0
+	    : ($N < 0)
+	      ? ceil (Ti($N))
+	      : floor(Ti($N));
+
+    print STDERR __FILE__, ' ', __LINE__, ' ', (caller(0))[3], ": ",
+	Data::Dumper->Dump([$n], [qw(n)]), "\n"
+	if DEBUG;
+    return $n;
+}
+
+#----------------------------------------------------------------------
+
+=head3 Tri($N)
+
+Returns right triangular index (integer) of $N.
+Note that Tri(0) = 0, and Tri(-$N) = -Tri($N).
+
+=cut
+
+sub Tri
+{
+    my $N = shift;
+    print STDERR __FILE__, ' ', __LINE__, ' ', (caller(0))[3], ": ",
+	Data::Dumper->Dump([$N], [qw(N)]), "\n"
+	if DEBUG;
+
+    my $n = ($N == 0)
+	    ? 0
+	    : ($N < 0)
+	      ? floor(Ti($N))
+	      : ceil (Ti($N));
+
+    print STDERR __FILE__, ' ', __LINE__, ' ', (caller(0))[3], ": ",
+	Data::Dumper->Dump([$n], [qw(n)]), "\n"
+	if DEBUG;
     return $n;
 }
 
@@ -181,29 +242,25 @@ sub Ti
 
 =head3 is_T($N)
 
-Returns true if $N is a Triangular number,
+Returns true if $N is a triangular number,
 if $N is zero,
-or if -$N is a Triangular number,
+or if -$N is a triangular number,
 otherwise returns false.
-Croaks if passed a non-integer argument.
 
 =cut
 
 sub is_T
 {
     my $N = shift;
-    print STDERR (caller(0))[3], ": ",
-	Data::Dumper->Dump([$N], [qw(x)]), "\n"
-	if $debug;
+    print STDERR __FILE__, ' ', __LINE__, ' ', (caller(0))[3], ": ",
+	Data::Dumper->Dump([$N], [qw(N)]), "\n"
+	if DEBUG;
 
-    croak "'$N' is not an integer"
-	unless $N =~ /^-?\d+$/;
+    my $r = (T(Tri($N)) == $N);
     
-    my $r = (T(Ti($N)) == $N);
-    
-    print STDERR (caller(0))[3], ": ",
+    print STDERR __FILE__, ' ', __LINE__, ' ', (caller(0))[3], ": ",
 	Data::Dumper->Dump([$r], [qw(r)]), "\n"
-	if $debug;
+	if DEBUG;
     return $r;
 }
 
@@ -230,18 +287,6 @@ To install this module type the following:
    make install
 
 
-=head1 DEPENDENCIES
-
-None.
-
-
-=head1 TO DO
-
-Everything is currently implemented using standard Perl scalars,
-so large arguments are likely to result in errors due to
-floating-point limitations.
-
-
 =head1 SEE ALSO
 
 http://mathforum.org/dr.math/faq/faq.pascal.triangle.html
@@ -253,18 +298,12 @@ http://en.wikipedia.org/wiki/Triangular_numbers
 
 =head1 AUTHOR
 
-David Christensen, <dpchrist@holgerdanske.com>
-
-
-=head1 CREDITS
-
-Rob "Sisyphus", for helping me find the correct term "index" and
-for suggesting a function to test if a number is triangular.
+David Paul Christensen dpchrist@holgerdanske.com
 
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2005 by David Christensen <dpchrist@holgerdanske.com>
+Copyright 2010 by David Paul Christensen dpchrist@holgerdanske.com
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.6 or,
